﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace InABox.Core
{
    public static class PosterUtils
    {
        private static Dictionary<Type, Type>? _posterEngines;

        private static Dictionary<Type, Type> GetPosterEngines()
        {
            _posterEngines ??= CoreUtils.TypeList(
                AppDomain.CurrentDomain.GetAssemblies(),
                x => x.IsClass
                    && !x.IsAbstract
                    && x.GenericTypeArguments.Length == 1
                    && x.HasInterface(typeof(IPosterEngine<>))
            ).ToDictionary(
                x => x.GetInterfaceDefinition(typeof(IPosterEngine<>))!.GenericTypeArguments[0],
                x => x);
            return _posterEngines;
        }

        public static Type GetEngine<T>()
            where T : Entity, IPostable
        {
            return GetPosterEngines().GetValueOrDefault(typeof(T))
                ?? throw new Exception($"No poster engine for type {typeof(T).Name}");
        }
        public static IPosterEngine<T> CreateEngine<T>()
            where T : Entity, IPostable
        {
            return (Activator.CreateInstance(GetEngine<T>()) as IPosterEngine<T>)!;
        }

        public static bool Process<T>(IEnumerable<T> entities)
            where T : Entity, IPostable
        {
            return CreateEngine<T>().Process(entities);
        }
    }
}
