using System;
using InABox.Clients;
using InABox.Core;

namespace InABox.Mobile
{
    public interface IEntityDocumentShell : IShell
    {
        Guid ID { get; }
        Guid ParentID { get; set; }
        Guid DocumentID { get; set; }
        String FileName { get; set; }
        byte[] Thumbnail { get; set; }
    }

    public static class EntityDocumentUtils
    {
        public static T SaveDocument<T>(MobileDocument image, Func<T> shell, string auditmessage) where T : IEntityDocumentShell
        {
            T result = default(T);
            Document doc = new Document()
            {
                FileName = image.FileName,
                Data = image.Data,
                CRC = CoreUtils.CalculateCRC(image.Data),
                TimeStamp = DateTime.Now
            };
            new Client<Document>().Save(doc, auditmessage);
                
            result = shell();
            result.DocumentID = doc.ID;
            result.FileName = doc.FileName;
            result.Thumbnail = MobileUtils.ImageTools.CreateThumbnail(doc.Data, 128, 128);
            result.Save(auditmessage);
        
            return result;
        }
    }
}